@@ -237,31 +237,21 @@ def _message_format(msg, image_info, device, partition_uuids):
     """Helper method to get and populate different messages."""
     message = None
     result_msg = msg
-    if image_info.get('image_type') == 'partition':
-        root_uuid = partition_uuids.get('root uuid')
-        efi_system_partition_uuid = (
-            partition_uuids.get('efi system partition uuid'))
-        if (image_info.get('deploy_boot_mode') == 'uefi'
-                and image_info.get('boot_option') == 'local'):
-            result_msg = msg + 'root_uuid={} efi_system_partition_uuid={}'
-            message = result_msg.format(image_info['id'], device,
-                                        root_uuid,
-                                        efi_system_partition_uuid)
-        else:
-            result_msg = msg + 'root_uuid={}'
-            message = result_msg.format(image_info['id'], device, root_uuid)
+
+    root_uuid = partition_uuids.get('root uuid')
+    efi_system_partition_uuid = (
+        partition_uuids.get('efi system partition uuid'))
+    if (image_info.get('deploy_boot_mode') == 'uefi'
+            and image_info.get('boot_option') == 'local'
+            and efi_system_partition_uuid):
+        result_msg = msg + 'root_uuid={} efi_system_partition_uuid={}'
+        message = result_msg.format(image_info['id'], device,
+                                    root_uuid,
+                                    efi_system_partition_uuid)
     else:
-        try:
-            # NOTE(TheJulia): ironic-lib disk_utils.get_disk_identifier
-            # can raise OSError if hexdump is not found.
-            root_uuid = disk_utils.get_disk_identifier(device)
-            result_msg = msg + 'root_uuid={}'
-            message = result_msg.format(image_info['id'], device, root_uuid)
-        except OSError as e:
-            LOG.warning('Failed to call get_disk_identifier: '
-                        'Unable to obtain the root_uuid parameter: '
-                        'The hexdump tool may be missing in IPA: %s', e)
-            message = result_msg.format(image_info['id'], device)
+        result_msg = msg + 'root_uuid={}'
+        message = result_msg.format(image_info['id'], device, root_uuid)
+
     return message
 
 
@@ -292,7 +282,6 @@ class ImageDownload(object):
                  any reason.
         """
         self._time = time_obj or time.time()
-        self._last_chunk_time = None
         self._image_info = image_info
         self._request = None
 
@@ -354,6 +343,7 @@ class ImageDownload(object):
         :returns: A chunk of the image. Size of chunk is IMAGE_CHUNK_SIZE
                   which is a constant in this module.
         """
+        self._last_chunk_time = None
         for chunk in self._request.iter_content(IMAGE_CHUNK_SIZE):
             # Per requests forum posts/discussions, iter_content should
             # periodically yield to the caller for the client to do things
@@ -606,6 +596,22 @@ class StandbyExtension(base.BaseAgentExtension):
             # Note: the catch internal to the helper method logs any errors.
             pass
 
+    def _fix_up_partition_uuids(self, image_info, device):
+        if self.partition_uuids is None:
+            self.partition_uuids = {}
+
+        if image_info.get('image_type') == 'partition':
+            return
+
+        try:
+            root_uuid = disk_utils.get_disk_identifier(device)
+        except OSError as e:
+            LOG.warning('Failed to call get_disk_identifier: '
+                        'Unable to obtain the root_uuid parameter: '
+                        'The hexdump tool may be missing in IPA: %s', e)
+        else:
+            self.partition_uuids['root uuid'] = root_uuid
+
     @base.async_command('cache_image', _validate_image_info)
     def cache_image(self, image_info=None, force=False):
         """Asynchronously caches specified image to the local OS device.
@@ -631,6 +637,7 @@ class StandbyExtension(base.BaseAgentExtension):
             self._cache_and_write_image(image_info, device)
             msg = 'image ({}) cached to device {} '
 
+        self._fix_up_partition_uuids(image_info, device)
         result_msg = _message_format(msg, image_info, device,
                                      self.partition_uuids)
 
@@ -679,6 +686,7 @@ class StandbyExtension(base.BaseAgentExtension):
                                                                   device)
                     stream_to = self.partition_uuids['partitions']['root']
                 else:
+                    self.partition_uuids = {}
                     stream_to = device
 
                 self._stream_raw_image_onto_device(image_info, stream_to)
@@ -699,6 +707,8 @@ class StandbyExtension(base.BaseAgentExtension):
                 disk_utils.create_config_drive_partition(node_uuid,
                                                          device,
                                                          configdrive)
+
+        self._fix_up_partition_uuids(image_info, device)
         msg = 'image ({}) written to device {} '
         result_msg = _message_format(msg, image_info, device,
                                      self.partition_uuids)
@@ -766,6 +776,16 @@ class StandbyExtension(base.BaseAgentExtension):
             LOG.error(error_msg)
             raise errors.CommandExecutionError(error_msg)
 
+    @base.sync_command('get_partition_uuids')
+    def get_partition_uuids(self):
+        """Return partition UUIDs."""
+        # NOTE(dtantsur): None means prepare_image hasn't been called (an empty
+        # dict is used for whole disk images).
+        if self.partition_uuids is None:
+            LOG.warning('No partition UUIDs recorded yet, prepare_image '
+                        'has to be called before get_partition_uuids')
+        return self.partition_uuids or {}
+
     # TODO(TheJulia): Once we have deploy/clean steps, this should
     # become a step, which we ideally have enabled by default.
     def _sync_clock(self, ignore_errors=False):
