@@ -37,6 +37,7 @@ import yaml
 
 from ironic_python_agent import encoding
 from ironic_python_agent import errors
+from ironic_python_agent.extensions import base as ext_base
 from ironic_python_agent import netutils
 from ironic_python_agent import raid_utils
 from ironic_python_agent import utils
@@ -132,34 +133,10 @@ def _check_for_iscsi():
                     "Error: %s", e)
 
 
-def _get_md_uuid(raid_device):
-    """Get the md UUID of a Software RAID device.
-
-    :param raid_device: A Software RAID block device name.
-    :returns: A string containing the UUID of an md device.
-    """
-    try:
-        out, _ = utils.execute('mdadm', '--detail', raid_device,
-                               use_standard_locale=True)
-    except processutils.ProcessExecutionError as e:
-        msg = ('Could not get the details of %(dev)s: %(err)s' %
-               {'dev': raid_device, 'err': e})
-        LOG.warning(msg)
-        return
-
-    lines = out.splitlines()
-    # the first line contains the md device itself
-    for line in lines[1:]:
-        match = re.search(r'UUID : ([a-f0-9:]+)', line)
-        if match:
-            return match.group(1)
-
-
 def _get_component_devices(raid_device):
     """Get the component devices of a Software RAID device.
 
-    Get the UUID of the md device and scan all other devices
-    for the same md UUID.
+    Examine an md device and return its constituent devices.
 
     :param raid_device: A Software RAID block device name.
     :returns: A list of the component devices.
@@ -167,35 +144,22 @@ def _get_component_devices(raid_device):
     if not raid_device:
         return []
 
-    md_uuid = _get_md_uuid(raid_device)
-    if not md_uuid:
+    try:
+        out, _ = utils.execute('mdadm', '--detail', raid_device,
+                               use_standard_locale=True)
+    except processutils.ProcessExecutionError as e:
+        msg = ('Could not get component devices of %(dev)s: %(err)s' %
+               {'dev': raid_device, 'err': e})
+        LOG.warning(msg)
         return []
-    LOG.debug('%s has UUID %s', raid_device, md_uuid)
 
     component_devices = []
-    block_devices = list_all_block_devices()
-    block_devices.extend(list_all_block_devices(block_type='part',
-                                                ignore_raid=True))
-    for bdev in block_devices:
-        try:
-            out, _ = utils.execute('mdadm', '--examine', bdev.name,
-                                   use_standard_locale=True)
-        except processutils.ProcessExecutionError as e:
-            if "No md superblock detected" in str(e):
-                # actually not a component device
-                LOG.debug('Not a component device %s', bdev.name)
-                continue
-            else:
-                LOG.warning("Failed to examine device %s: %s",
-                            bdev.name, e)
-                continue
-        lines = out.splitlines()
-        for line in lines:
-            if md_uuid in line:
-                component_devices.append(bdev.name)
+    lines = out.splitlines()
+    # the first line contains the md device itself
+    for line in lines[1:]:
+        device = re.findall(r'/dev/\w+', line)
+        component_devices += device
 
-    LOG.info('Found component devices for %s:',
-             raid_device, component_devices)
     return component_devices
 
 
@@ -207,16 +171,16 @@ def _calc_memory(sys_dict):
         for core_child in sys_child['children']:
             if not _MEMORY_ID_RE.match(core_child['id']):
                 continue
-            if core_child.get('size'):
+            if (not core_child.get("children")
+                    and core_child.get('size')):
                 value = ("%(size)s %(units)s" % core_child)
                 physical += int(UNIT_CONVERTER(value).to
                                 ('MB').magnitude)
-            else:
-                for bank in core_child.get('children', ()):
-                    if bank.get('size'):
-                        value = ("%(size)s %(units)s" % bank)
-                        physical += int(UNIT_CONVERTER(value).to
-                                        ('MB').magnitude)
+            for bank in core_child.get('children', ()):
+                if bank.get('size'):
+                    value = ("%(size)s %(units)s" % bank)
+                    physical += int(UNIT_CONVERTER(value).to
+                                    ('MB').magnitude)
     return physical
 
 
@@ -330,8 +294,7 @@ def _md_scan_and_assemble():
 
 def list_all_block_devices(block_type='disk',
                            ignore_raid=False,
-                           ignore_floppy=True,
-                           ignore_empty=True):
+                           ignore_floppy=True):
     """List all physical block devices
 
     The switches we use for lsblk: P for KEY="value" output, b for size output
@@ -347,7 +310,6 @@ def list_all_block_devices(block_type='disk',
                         devices and should be treated as such if encountered.
     :param ignore_floppy: Ignore floppy disk devices in the block device
                           list. By default, these devices are filtered out.
-    :param ignore_empty: Whether to ignore disks with size equal 0.
     :return: A list of BlockDevices
     """
 
@@ -416,24 +378,12 @@ def list_all_block_devices(block_type='disk',
         # Other possible type values, which we skip recording:
         #   lvm, part, rom, loop
         if devtype != block_type:
-            if (devtype is None or ignore_raid):
-                LOG.debug("Skipping: {!r}".format(line))
-                continue
-            elif ('raid' in devtype
-                  and block_type in ['raid', 'disk']):
+            if (devtype is not None
+                    and any(x in devtype for x in ['raid', 'md'])
+                    and not ignore_raid):
                 LOG.debug(
-                    "TYPE detected to contain 'raid', signifying a "
+                    "TYPE detected to contain 'raid' or 'md', signifying a "
                     "RAID volume. Found: {!r}".format(line))
-            elif (devtype == 'md'
-                  and (block_type == 'part'
-                       or block_type == 'md')):
-                # NOTE(dszumski): Partitions on software RAID devices have type
-                # 'md'. This may also contain RAID devices in a broken state in
-                # rare occasions. See https://review.opendev.org/#/c/670807 for
-                # more detail.
-                LOG.debug(
-                    "TYPE detected to contain 'md', signifying a "
-                    "RAID partition. Found: {!r}".format(line))
             else:
                 LOG.debug(
                     "TYPE did not match. Wanted: {!r} but found: {!r}".format(
@@ -453,12 +403,6 @@ def list_all_block_devices(block_type='disk',
             LOG.debug('Skipping RAM device %s', device)
             continue
 
-        # NOTE(dtantsur): some hardware represents virtual floppy devices as
-        # normal block devices with size 0. Filter them out.
-        if ignore_empty and not int(device['SIZE'] or 0):
-            LOG.debug('Skipping device %s with zero size', device)
-            continue
-
         name = os.path.join('/dev', device['KNAME'])
 
         try:
@@ -1063,13 +1007,14 @@ class GenericHardwareManager(HardwareManager):
             total = None
             LOG.exception(("Cannot fetch total memory size using psutil "
                            "version %s"), psutil.version_info[0])
-        sys_dict = None
         try:
             sys_dict = _get_system_lshw_dict()
         except (processutils.ProcessExecutionError, OSError, ValueError) as e:
             LOG.warning('Could not get real physical RAM from lshw: %s', e)
             physical = None
         else:
+            if isinstance(sys_dict, str):
+                sys_dict = json.loads(sys_dict)
             physical = _calc_memory(sys_dict)
 
             if not physical:
@@ -1625,6 +1570,15 @@ class GenericHardwareManager(HardwareManager):
                 'reboot_requested': False,
                 'argsinfo': RAID_APPLY_CONFIGURATION_ARGSINFO,
             },
+            {
+                'step': 'write_image',
+                # NOTE(dtantsur): this step has to be proxied via an
+                # out-of-band step with the same name, hence the priority here
+                # doesn't really matter.
+                'priority': 0,
+                'interface': 'deploy',
+                'reboot_requested': False,
+            },
         ]
 
     def apply_configuration(self, node, ports, raid_config,
@@ -1640,7 +1594,7 @@ class GenericHardwareManager(HardwareManager):
         self.validate_configuration(raid_config, node)
         if delete_existing:
             self.delete_configuration(node, ports)
-        return self._do_create_configuration(node, ports, raid_config)
+        self._do_create_configuration(node, ports, raid_config)
 
     def create_configuration(self, node, ports):
         """Create a RAID configuration.
@@ -1711,21 +1665,8 @@ class GenericHardwareManager(HardwareManager):
         partition_table_type = utils.get_partition_table_type_from_specs(node)
         target_boot_mode = utils.get_node_boot_mode(node)
 
-        parted_start_dict = {}
-        # Create a partition table on each disk.
-        for dev_name in block_devices:
-            LOG.info("Creating partition table on {}".format(
-                dev_name))
-            try:
-                utils.execute('parted', dev_name, '-s', '--',
-                              'mklabel', partition_table_type)
-            except processutils.ProcessExecutionError as e:
-                msg = "Failed to create partition table on {}: {}".format(
-                    dev_name, e)
-                raise errors.SoftwareRAIDError(msg)
-
-            parted_start_dict[dev_name] = raid_utils.calculate_raid_start(
-                target_boot_mode, partition_table_type, dev_name)
+        parted_start_dict = raid_utils.create_raid_partition_tables(
+            block_devices, partition_table_type, target_boot_mode)
 
         LOG.debug("First available sectors per devices %s", parted_start_dict)
 
@@ -1780,7 +1721,7 @@ class GenericHardwareManager(HardwareManager):
                     # The real difference between partx and partprobe is
                     # unclear, but note that partprobe does not seem to
                     # work synchronously for nvme drives...
-                    utils.execute("partx", "-a", device,
+                    utils.execute("partx", "-u", device,
                                   check_exit_code=False)
                 except processutils.ProcessExecutionError as e:
                     msg = "Failed to create partitions on {}: {}".format(
@@ -1839,18 +1780,7 @@ class GenericHardwareManager(HardwareManager):
             utils.execute('mdadm', '--assemble', '--scan',
                           check_exit_code=False)
             raid_devices = list_all_block_devices(block_type='raid',
-                                                  ignore_raid=False,
-                                                  ignore_empty=False)
-            # NOTE(dszumski): Fetch all devices of type 'md'. This
-            # will generally contain partitions on a software RAID
-            # device, but crucially may also contain devices in a
-            # broken state. See https://review.opendev.org/#/c/670807/
-            # for more detail.
-            raid_devices.extend(
-                list_all_block_devices(block_type='md',
-                                       ignore_raid=False,
-                                       ignore_empty=False)
-            )
+                                                  ignore_raid=False)
             return raid_devices
 
         raid_devices = _scan_raids()
@@ -1868,7 +1798,6 @@ class GenericHardwareManager(HardwareManager):
             raise errors.SoftwareRAIDError(msg)
 
     def _delete_config_pass(self, raid_devices):
-        all_holder_disks = []
         for raid_device in raid_devices:
             component_devices = _get_component_devices(raid_device.name)
             if not component_devices:
@@ -1921,11 +1850,14 @@ class GenericHardwareManager(HardwareManager):
                     LOG.warning('Failed to remove superblock from %s: %s',
                                 raid_device.name, e)
 
-            # NOTE(arne_wiebalck): We cannot delete the partitions right
-            # away since there may be other partitions on the same disks
-            # which are members of other RAID devices. So we remember them
-            # for later.
-            all_holder_disks.extend(holder_disks)
+            # Remove the partitions we created during create_configuration.
+            for holder_disk in holder_disks:
+                LOG.debug('Removing partitions on %s', holder_disk)
+                try:
+                    utils.execute('wipefs', '-af', holder_disk)
+                except processutils.ProcessExecutionError:
+                    LOG.warning('Failed to remove partitions on %s',
+                                holder_disk)
 
             LOG.info('Deleted Software RAID device %s', raid_device.name)
 
@@ -1971,17 +1903,6 @@ class GenericHardwareManager(HardwareManager):
                 LOG.warning('Failed to remove superblock from %s: %s',
                             raid_device.name, e)
 
-        # Erase all partition tables we created
-        all_holder_disks_uniq = list(
-            collections.OrderedDict.fromkeys(all_holder_disks))
-        for holder_disk in all_holder_disks_uniq:
-            LOG.info('Removing partitions on holder disk %s', holder_disk)
-            try:
-                utils.execute('wipefs', '-af', holder_disk)
-            except processutils.ProcessExecutionError as e:
-                LOG.warning('Failed to remove partitions on %s',
-                            holder_disk, e)
-
         LOG.debug("Finished deleting Software RAID(s)")
 
     def validate_configuration(self, raid_config, node):
@@ -2072,6 +1993,25 @@ class GenericHardwareManager(HardwareManager):
                                       'errors': '; '.join(raid_errors)}
             raise errors.SoftwareRAIDError(error)
 
+    def write_image(self, node, ports, image_info, configdrive=None):
+        """A deploy step to write an image.
+
+        Downloads and writes an image to disk if necessary. Also writes a
+        configdrive to disk if the configdrive parameter is specified.
+
+        :param node: A dictionary of the node object
+        :param ports: A list of dictionaries containing information
+                      of ports for the node
+        :param image_info: Image information dictionary.
+        :param configdrive: A string containing the location of the config
+                            drive as a URL OR the contents (as gzip/base64)
+                            of the configdrive. Optional, defaults to None.
+        """
+        ext = ext_base.get_extension('standby')
+        cmd = ext.prepare_image(image_info=image_info, configdrive=configdrive)
+        # The result is asynchronous, wait here.
+        cmd.join()
+
 
 def _compare_extensions(ext1, ext2):
     mgr1 = ext1.obj
